from .node import Node
import re


class Style(object):
    def __init__(self, selector, **kwargs):
        self.selector = selector
        self.params = kwargs

    def to_string(self, indent=0):
        out_str = f"{' '*indent}{self.selector} {{\n"

        keys = list(self.params.keys())
        keys.sort()
        out_str += ''.join([(' '*(indent) + self._param_str(k)) for k in keys])

        out_str += f"{' '*indent}}}"
        out_str = out_str.encode('utf-8')

        return out_str

    def __getitem__(self, key):
        return self.params[key]

    def __setitem__(self, key, item):
        self.params[key] = item

    def __str__(self):
        return self.to_string().decode('utf-8')

    def _param_str(self, key):
        return f"  {re.sub(r'_', '-', key)}: {self.params[key]};\n"


if (__name__ == '__main__'):
    style = Style('.my-class', fill='orange', stroke='black', stroke_width=1.2)

    print(style)



import re


class Rotation(object):
    def __init__(self, deg, center=None):
        self.deg = deg
        self.center = center

    def __str__(self):
        params = [self.deg]
        params += list(self.center or [])
        return f"rotate({','.join([str(p) for p in params])})"


class Scale(object):
    def __init__(self, xy):
        self.x = xy[0]
        self.y = xy[1]

    def __str__(self):
        return f"scale({self.x},{self.y})"


class Skew(object):
    def __init__(self, val):
        self.val = val

    def __str__(self):
        return f"skew{self.axis}({self.val})"


class SkewX(Skew):
    axis = 'X'


class SkewY(Skew):
    axis = 'Y'


class Translation(object):
    def __init__(self, xy):
        self.x = xy[0]
        self.y = xy[1]

    def __str__(self):
        return f"translate({self.x} {self.y})"


class Node(object):
    def __init__(self, tag, **kwargs):
        self.tag = tag
        self.params = kwargs
        self.children = []
        self.classes = []
        self.transforms = []

    @property
    def content(self):
        return ''

    @property
    def num_children(self):
        return len(self.children)

    def add_title(self, title):
        self.append_child(TitleNode(title))
        return self

    def append_child(self, node):
        self.children.append(node)
        return self

    def insert_child(self, index, node):
        self.children.insert(index, node)
        return self

    def delete_child(self, index):
        del self.children[index]
        return self

    def append_class(self, cls):
        self.classes.append(cls)
        return self

    def insert_class(self, index, cls):
        self.classes.insert(index, cls)
        return self

    def remove_class(self, cls):
        self.classes.remove(cls)
        return self

    def rotate(self, deg, center=None):
        self.transforms.append(Rotation(deg, center))
        return self

    def scale(self, xy):
        self.transforms.append(Scale(xy))
        return self

    def skew_x(self, val):
        self.transforms.append(SkewX(val))
        return self

    def skew_y(self, val):
        self.transforms.append(SkewY(val))
        return self

    def translate(self, xy):
        self.transforms.append(Translation(xy))
        return self

    def to_string(self, indent=0):
        out_str = f"{' '*indent}<{self.tag}"

        params = dict(self.params)
        if len(self.classes) != 0:
            params['class'] = ' '.join([c for c in self.classes])

        if len(self.transforms) != 0:
            params['transform'] = ' '.join([str(t) for t in self.transforms])

        keys = list(params.keys())
        if len(keys) > 0:
            out_str += ' '
            keys.sort()
            out_str += ' '.join(
                [f'{re.sub(r"_", "-", k)}="{params[k]}"' for k in keys]
            ).strip()

        out_str = out_str.encode('utf-8')

        if len(self.children) == 0 and len(self.content) == 0:
            out_str += '/>\n'.encode('utf-8')
        else:
            out_str += '>\n'.encode('utf-8')
            if len(self.content) != 0:
                indented_content = self.content.decode('utf-8').split('\n')
                indented_content = \
                    [' '*(indent+2) + c for c in indented_content]
                out_str += '\n'.join(indented_content).encode('utf-8')
                out_str += '\n'.encode('utf-8')
            else:
                for c in self.children:
                    out_str += c.to_string(indent=(indent+2))
            out_str += f"{' '*indent}</{self.tag}>\n".encode('utf-8')

        return out_str

    def __str__(self):
        return self.to_string().decode('utf-8')


class TitleNode(Node):
    def __init__(self, title):
        super().__init__('title')
        self.title = title

    @property
    def content(self):
        return self.title.encode('utf-8')



from .node import Node


class Path(Node):
    def __init__(self, starting_xy, **kwargs):
        super().__init__("path", **kwargs)
        self.commands = []
        self.move_to(starting_xy)

    def to_string(self, indent=0):
        self.params['d'] = ' '.join(self.commands)
        return super().to_string(indent)

    def arc_to(self, rx_ry, x_rotation, large_arc,
               sweep_ccw, end_xy, relative=False):
        params = list(rx_ry)
        params.append(x_rotation)
        params.append('1' if large_arc else '0')
        params.append('1' if sweep_ccw else '0')
        params += list(end_xy)
        self._append_command('A', relative, *params)
        return self

    def c_bezier(self, ctrl1, ctrl2, end, relative=False):
        if ctrl1:
            coord_str = f"{ctrl1[0]} {ctrl1[1]}, "
            cmd = 'C'
        else:
            coord_str = ''
            cmd = 'S'
        coord_str += (f"{ctrl2[0]} {ctrl2[1]}, " +
                      f"{end[0]} {end[1]}")
        self._append_command(cmd, relative, coord_str)
        return self

    def close(self):
        self._append_command('Z')
        return self

    def h_line(self, x, relative=False):
        self._append_command('H', relative, x)
        return self

    def line_to(self, xy, relative=False):
        self._append_command('L', relative, *(list(xy)))
        return self

    def move_to(self, xy, relative=False):
        self._append_command('M', relative, *(list(xy)))
        return self

    def q_bezier(self, ctrl, end, relative=False):
        if ctrl:
            coord_str = f"{ctrl[0]} {ctrl[1]}, "
            cmd = 'Q'
        else:
            coord_str = ''
            cmd = 'T'
        coord_str += f"{end[0]} {end[1]}"
        self._append_command(cmd, relative, coord_str)
        return self

    def v_line(self, y, relative=False):
        self._append_command('V', relative, y)
        return self

    def _append_command(self, cmd, relative=False, *args):
        if relative:
            cmd = cmd.lower()
        self.commands.append(' '.join([cmd] + [str(a) for a in args]))


if (__name__ == '__main__'):
    from svg import RootSvg, Rectangle

    svg = RootSvg((-25, -25, 50, 50), stroke='black', width='50%')
    svg.append_child(
        Rectangle(p1=(-50, -50), p2=(50, 50), fill='#cccccc', stroke='none'))
    svg.append_child(
        Rectangle(center=(0, 0), width=1, height=1, fill='red', stroke='none'))
    path = Path((6.7, -10), stroke_width=0.2, fill='none')
    svg.append_child(path)

    path.line_to((0, 20), relative=True)
    path.h_line(-2, relative=True)
    path.arc_to((12.9, 12.9), 0, False, True, (0, -20), relative=True)
    path.close()

    print(svg)
    svg.write('src/svg/path.html')



from .node import Node
from .style import Style


class Svg(Node):
    def __init__(self, view_box, **kwargs):
        view_box_str = ' '.join([str(i) for i in list(view_box)])
        super().__init__("svg", viewBox=view_box_str, **kwargs)


class StyleNode(Node):
    def __init__(self, styles):
        super().__init__("style")
        self.styles = styles

    @property
    def content(self):
        content = b''
        for i, s in enumerate(self.styles):
            if i > 0:
                content += '\n'.encode('utf-8')
            content += s.to_string()
        return content


class RootSvg(Svg):
    def __init__(self, view_box, **kwargs):
        view_box_str = ' '.join([str(i) for i in list(view_box)])
        super().__init__(view_box,
                         xmlns="http://www.w3.org/2000/svg",
                         **kwargs)
        self.styles = []

    def add_style(self, style):
        self.styles.append(style)

    def to_string(self, indent=0, omit_styles=False):
        omit_styles = omit_styles or len(self.styles) == 0
        if not omit_styles:
            self.insert_child(0, StyleNode(self.styles))

        string = super().to_string(indent=indent)

        if not omit_styles:
            self.delete_child(0)

        return string

    def write(self, filename):
        f = open(filename, 'wb')
        f.write(self.to_string())
        f.close()


class Circle(Node):
    def __init__(self, center, radius, **kwargs):
        super().__init__("circle",
                         cx=center[0],
                         cy=center[1],
                         r=radius,
                         **kwargs)


class Ellipse(Node):
    def __init__(self, center, dims, **kwargs):
        super().__init__("ellipse",
                         cx=center[0],
                         cy=center[1],
                         rx=dims[0],
                         ry=dims[1],
                         **kwargs)


class Group(Node):
    def __init__(self, children=[], **kwargs):
        super().__init__("g", **kwargs)
        if not isinstance(children, list):
            children = [children]
        for c in children:
            self.append_child(c)


class Line(Node):
    def __init__(self, p1, p2, **kwargs):
        super().__init__("line",
                         x1=p1[0],
                         y1=p1[1],
                         x2=p2[0],
                         y2=p2[1],
                         **kwargs)


class Poly(Node):
    def __init__(self, tag, points, **kwargs):
        points_str = ' '.join([f"{p[0]},{p[1]}" for p in points])
        super().__init__(tag, points=points_str, **kwargs)


class Polygon(Poly):
    def __init__(self, points, **kwargs):
        super().__init__("polygon", points, **kwargs)


class PolyLine(Poly):
    def __init__(self, points, **kwargs):
        super().__init__("polyline", points, **kwargs)


class Rectangle(Node):
    def __init__(self, p1=None, p2=None, center=None,
                 width=None, height=None, radius=None, **kwargs):
        if center and width and height:
            x1 = center[0] - width / 2.0
            y1 = center[1] - height / 2.0
        else:
            x1 = p1[0]
            y1 = p1[1]
            width = width or (p2[0] - p1[0])
            height = height or (p2[1] - p1[1])

        if radius:
            if isinstance(radius, tuple):
                kwargs['rx'] = radius[0]
                kwargs['ry'] = radius[1]
            else:
                kwargs['rx'] = radius

        super().__init__("rect",
                         x=x1,
                         y=y1,
                         width=width,
                         height=height,
                         **kwargs)


class Text(Node):
    def __init__(self, xy, text, **kwargs):
        super().__init__("text", x=xy[0], y=xy[1], **kwargs)
        self.text = text

    @property
    def content(self):
        return self.text.encode('utf-8')


if (__name__ == '__main__'):
    svg = RootSvg((0, 0, 300, 100), stroke="purple")
    svg.add_style(Style('.my-class', stroke='gray', stroke_width=10))
    svg.add_style(Style('.other-class', fill='pink'))
    group = Group(Circle((50, 50), 40), stroke='green')
    svg.append_child(group)
    circ = Circle((150, 50), 4)
    circ.add_title('this is a little circle')
    group.append_child(circ)
    group.translate((5, 7))
    child_svg = Svg((0, 0, 10, 10), x=200, width=100)
    child_svg.append_child(Circle((5, 5), 4))
    svg.append_child(child_svg)

    line = Line((5, 15), (27, 10), stroke="blue")
    svg.append_child(line)
    line.rotate(5, (10, 10))

    line = Line((130, 50), (27, 10))
    svg.append_child(line)
    line.append_class('my-class')

    txt = Text((10, 40), 'this is my text', fill='green')
    svg.add_style(Style('.wacky-text',
                        font_family='Comic Sans MS',
                        stroke='none',
                        fill='pink'))
    svg.add_style(Style('.gigantic', font_size=50))
    txt.append_class('wacky-text')
    txt.append_class('gigantic')
    svg.append_child(txt)

    print(svg)
    svg.write('src/svg/my.html')



from setuptools import setup
from os import path

# Read the contents of the README file
cwd = path.abspath(path.dirname(__file__))
with open(path.join(cwd, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="pysvglib",
    version="0.3.2",
    description="SVG drawing library",
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Multimedia :: Graphics',
    ],
    keywords='svg graphics',
    url='https://github.com/gbingersoll/pysvglib',
    author='Greg Ingersoll',
    author_email='greg.ingersoll@convolutionresearch.com',
    license='MIT',
    packages=['svg'],
    extras_require={
        'dev': ['pytest', 'pycodestyle', 'setuptools', 'wheel']
    }
)



import re


def underscore(str):
    # Find a capital letter followed by a string of lowercase letters
    # skipping the beginning of the string.  Insert an underscore.
    s = re.sub(r'(.)([A-Z][a-z]+)', r'\1_\2', str)

    # Find a lowercase letter or number followed by a capital letter,
    # and insert an underscore.  Convert all to lowercase.
    return re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', s).lower()


def css_name(str):
    return re.sub(r'_', '-', underscore(str))
